@@ -36,6 +36,7 @@
 #include "logging.h"
 #include "lkowner.h"
 #include "compat-uuid.h"
+#include "refcount.h"
 
 #define GF_YES 1
 #define GF_NO  0
@@ -82,7 +83,9 @@
 #define GF_XATTR_CLRLK_CMD      "glusterfs.clrlk"
 #define GF_XATTR_PATHINFO_KEY   "trusted.glusterfs.pathinfo"
 #define GF_XATTR_NODE_UUID_KEY  "trusted.glusterfs.node-uuid"
+#define GF_XATTR_LIST_NODE_UUIDS_KEY "trusted.glusterfs.list-node-uuids"
 #define GF_REBAL_FIND_LOCAL_SUBVOL "glusterfs.find-local-subvol"
+#define GF_REBAL_OLD_FIND_LOCAL_SUBVOL "glusterfs.old-find-local-subvol"
 #define GF_XATTR_VOL_ID_KEY   "trusted.glusterfs.volume-id"
 #define GF_XATTR_LOCKINFO_KEY   "trusted.glusterfs.lockinfo"
 #define GF_META_LOCK_KEY        "glusterfs.lock-migration-meta-lock"
@@ -90,6 +93,7 @@
 #define GF_XATTR_GET_REAL_FILENAME_KEY "glusterfs.get_real_filename:"
 #define GF_XATTR_USER_PATHINFO_KEY   "glusterfs.pathinfo"
 #define GF_INTERNAL_IGNORE_DEEM_STATFS "ignore-deem-statfs"
+#define GF_XATTR_IOSTATS_DUMP_KEY "trusted.io-stats-dump"
 
 #define GF_READDIR_SKIP_DIRS       "readdir-filter-directories"
 #define GF_MDC_LOADED_KEY_NAMES     "glusterfs.mdc.loaded.key.names"
@@ -104,6 +108,8 @@
                                          strlen (x)) == 0))
 #define XATTR_IS_NODE_UUID(x) (strncmp (x, GF_XATTR_NODE_UUID_KEY,      \
                                         strlen (GF_XATTR_NODE_UUID_KEY)) == 0)
+#define XATTR_IS_NODE_UUID_LIST(x) (strncmp (x, GF_XATTR_LIST_NODE_UUIDS_KEY, \
+                                    strlen (GF_XATTR_LIST_NODE_UUIDS_KEY)) == 0)
 #define XATTR_IS_LOCKINFO(x) (strncmp (x, GF_XATTR_LOCKINFO_KEY,        \
                                        strlen (GF_XATTR_LOCKINFO_KEY)) == 0)
 
@@ -169,6 +175,9 @@
 #define VIRTUAL_QUOTA_XATTR_CLEANUP_KEY "glusterfs.quota-xattr-cleanup"
 #define QUOTA_READ_ONLY_KEY "trusted.glusterfs.quota.read-only"
 
+/* afr related */
+#define AFR_XATTR_PREFIX "trusted.afr"
+
 /* Index xlator related */
 #define GF_XATTROP_INDEX_GFID "glusterfs.xattrop_index_gfid"
 #define GF_XATTROP_ENTRY_CHANGES_GFID "glusterfs.xattrop_entry_changes_gfid"
@@ -230,9 +239,12 @@
 #define GF_MAX_AUX_GROUPS   65535
 
 #define GF_UUID_BUF_SIZE 50
+#define GF_UUID_BNAME_BUF_SIZE (320) /* (64 + 256) */
 
 #define GF_REBALANCE_TID_KEY     "rebalance-id"
 #define GF_REMOVE_BRICK_TID_KEY  "remove-brick-id"
+#define GF_TIER_TID_KEY          "tier-id"
+#define GF_TIER_ENABLED          "tier-enabled"
 
 #define UUID_CANONICAL_FORM_LEN  36
 
@@ -395,6 +407,11 @@ struct _cmd_args {
 #ifdef GF_LINUX_HOST_OS
         char            *oom_score_adj;
 #endif
+
+        /* Run this process with valgrind? Might want to prevent calling
+         * functions that prevent valgrind from working correctly, like
+         * dlclose(). */
+        int             valgrind;
 };
 typedef struct _cmd_args cmd_args_t;
 
@@ -426,6 +443,12 @@ typedef enum {
 
 struct tvec_base;
 
+/* reference counting for the global (per ctx) timer-wheel */
+struct gf_ctx_tw {
+        GF_REF_DECL;
+        struct tvec_base *timer_wheel; /* global timer-wheel instance */
+};
+
 struct _glusterfs_ctx {
         cmd_args_t          cmd_args;
         char               *process_uuid;
@@ -494,14 +517,15 @@ struct _glusterfs_ctx {
          */
         mgmt_ssl_t          secure_srvr;
         /* Buffer to 'save' backtrace even under OOM-kill like situations*/
-        char btbuf[GF_BACKTRACE_LEN];
+        char                btbuf[GF_BACKTRACE_LEN];
 
-        pthread_mutex_t notify_lock;
-        pthread_cond_t notify_cond;
-        int notifying;
+        pthread_mutex_t     notify_lock;
+        pthread_cond_t      notify_cond;
+        int                 notifying;
 
-        struct tvec_base *timer_wheel; /* global timer-wheel instance */
+        struct gf_ctx_tw   *tw; /* refcounted timer_wheel */
 
+        gf_lock_t           volfile_lock;
 };
 typedef struct _glusterfs_ctx glusterfs_ctx_t;
 
@@ -551,16 +575,21 @@ typedef struct lock_migration_info {
  */
 #define SECURE_ACCESS_FILE     GLUSTERD_DEFAULT_WORKDIR "/secure-access"
 
-int glusterfs_graph_prepare (glusterfs_graph_t *graph, glusterfs_ctx_t *ctx);
+int glusterfs_graph_prepare (glusterfs_graph_t *graph, glusterfs_ctx_t *ctx,
+                             char *volume_name);
 int glusterfs_graph_destroy_residual (glusterfs_graph_t *graph);
 int glusterfs_graph_deactivate (glusterfs_graph_t *graph);
 int glusterfs_graph_destroy (glusterfs_graph_t *graph);
 int glusterfs_get_leaf_count (glusterfs_graph_t *graph);
 int glusterfs_graph_activate (glusterfs_graph_t *graph, glusterfs_ctx_t *ctx);
 glusterfs_graph_t *glusterfs_graph_construct (FILE *fp);
-glusterfs_graph_t *glusterfs_graph_new ();
+int glusterfs_graph_init (glusterfs_graph_t *graph);
+glusterfs_graph_t *glusterfs_graph_new (void);
 int glusterfs_graph_reconfigure (glusterfs_graph_t *oldgraph,
                                   glusterfs_graph_t *newgraph);
+int glusterfs_graph_attach (glusterfs_graph_t *orig_graph, char *path,
+                            glusterfs_graph_t **newgraph);
+int glusterfs_graph_parent_up (glusterfs_graph_t *graph);
 
 void
 gf_free_mig_locks (lock_migration_info_t *locks);
