@@ -29,16 +29,17 @@ from gconf import gconf
 from syncdutils import FreeObject, norm, grabpidfile, finalize
 from syncdutils import log_raise_exception, privileged, boolify
 from syncdutils import GsyncdError, select, set_term_handler
-from configinterface import GConffile, upgrade_config_file
+from configinterface import GConffile, upgrade_config_file, TMPL_CONFIG_FILE
 import resource
 from monitor import monitor
 import xml.etree.ElementTree as XET
 from subprocess import PIPE
 import subprocess
 from changelogagent import agent, Changelog
-from gsyncdstatus import set_monitor_status, GeorepStatus
+from gsyncdstatus import set_monitor_status, GeorepStatus, human_time_utc
 from libcxattr import Xattr
 import struct
+from syncdutils import get_master_and_slave_data_from_args
 
 ParseError = XET.ParseError if hasattr(XET, 'ParseError') else SyntaxError
 
@@ -244,6 +245,7 @@ def main_i():
                   default=os.devnull, type=str, action='callback',
                   callback=store_abs)
     op.add_option('--gluster-log-level', metavar='LVL')
+    op.add_option('--changelog-log-level', metavar='LVL', default="INFO")
     op.add_option('--gluster-params', metavar='PRMS', default='')
     op.add_option(
         '--glusterd-uuid', metavar='UUID', type=str, default='',
@@ -267,6 +269,7 @@ def main_i():
                   type=str, action='callback', callback=store_abs)
     op.add_option('--georep-session-working-dir', metavar='STATF',
                   type=str, action='callback', callback=store_abs)
+    op.add_option('--access-mount', default=False, action='store_true')
     op.add_option('--ignore-deletes', default=False, action='store_true')
     op.add_option('--isolated-slave', default=False, action='store_true')
     op.add_option('--use-rsync-xattrs', default=False, action='store_true')
@@ -274,6 +277,7 @@ def main_i():
     op.add_option('--sync-acls', default=True, action='store_true')
     op.add_option('--log-rsync-performance', default=False,
                   action='store_true')
+    op.add_option('--max-rsync-retries', type=int, default=10)
     op.add_option('--pause-on-start', default=False, action='store_true')
     op.add_option('-L', '--log-level', metavar='LVL')
     op.add_option('-r', '--remote-gsyncd', metavar='CMD',
@@ -282,6 +286,10 @@ def main_i():
     op.add_option('--slave-id', metavar='ID')
     op.add_option('--session-owner', metavar='ID')
     op.add_option('--local-id', metavar='ID', help=SUPPRESS_HELP, default='')
+    op.add_option(
+        '--local-node', metavar='NODE', help=SUPPRESS_HELP, default='')
+    op.add_option(
+        '--local-node-id', metavar='NODEID', help=SUPPRESS_HELP, default='')
     op.add_option(
         '--local-path', metavar='PATH', help=SUPPRESS_HELP, default='')
     op.add_option('-s', '--ssh-command', metavar='CMD', default='ssh')
@@ -407,7 +415,8 @@ def main_i():
                 if (o.callback in (store_abs, 'store_true', None) and
                     o.get_opt_string() not in ('--version', '--help'))]
     remote_tunables = ['listen', 'go_daemon', 'timeout',
-                       'session_owner', 'config_file', 'use_rsync_xattrs']
+                       'session_owner', 'config_file', 'use_rsync_xattrs',
+                       'local_id', 'local_node', 'access_mount']
     rq_remote_tunables = {'listen': True}
 
     # precedence for sources of values: 1) commandline, 2) cfg file, 3)
@@ -535,16 +544,19 @@ def main_i():
                 namedict[name + 'vol'] = x.volume
                 if name == 'remote':
                     namedict['remotehost'] = x.remotehost
-    if not 'config_file' in rconf:
-        rconf['config_file'] = os.path.join(
-            os.path.dirname(sys.argv[0]), "conf/gsyncd_template.conf")
 
-    upgrade_config_file(rconf['config_file'])
+    if not 'config_file' in rconf:
+        rconf['config_file'] = TMPL_CONFIG_FILE
+
+    # Upgrade Config File only if it is session conf file
+    if rconf['config_file'] != TMPL_CONFIG_FILE:
+        upgrade_config_file(rconf['config_file'], confdata)
+
     gcnf = GConffile(
-        rconf['config_file'], canon_peers,
+        rconf['config_file'], canon_peers, confdata,
         defaults.__dict__, opts.__dict__, namedict)
 
-    checkpoint_change = False
+    conf_change = False
     if confdata:
         opt_ok = norm(confdata.opt) in tunables + [None]
         if confdata.op == 'check':
@@ -563,10 +575,10 @@ def main_i():
         # when modifying checkpoint, it's important to make a log
         # of that, so in that case we go on to set up logging even
         # if its just config invocation
-        if confdata.opt == 'checkpoint' and confdata.op in ('set', 'del') and \
-           not confdata.rx:
-            checkpoint_change = True
-        if not checkpoint_change:
+        if confdata.op in ('set', 'del') and not confdata.rx:
+            conf_change = True
+
+        if not conf_change:
             return
 
     gconf.__dict__.update(defaults.__dict__)
@@ -664,13 +676,23 @@ def main_i():
     if not privileged() and gconf.log_file_mbr:
         gconf.log_file = gconf.log_file_mbr
 
-    if checkpoint_change:
+    if conf_change:
         try:
             GLogger._gsyncd_loginit(log_file=gconf.log_file, label='conf')
+            gconf.log_exit = False
+
             if confdata.op == 'set':
-                logging.info('checkpoint %s set' % confdata.val)
+                if confdata.opt == 'checkpoint':
+                    logging.info("Checkpoint Set: %s" % (
+                        human_time_utc(confdata.val)))
+                else:
+                    logging.info("Config Set: %s = %s" % (
+                        confdata.opt, confdata.val))
             elif confdata.op == 'del':
-                logging.info('checkpoint info was reset')
+                if confdata.opt == 'checkpoint':
+                    logging.info("Checkpoint Reset")
+                else:
+                    logging.info("Config Reset: %s" % confdata.opt)
         except IOError:
             if sys.exc_info()[1].errno == ENOENT:
                 # directory of log path is not present,
@@ -687,6 +709,17 @@ def main_i():
     if create:
         if getattr(gconf, 'state_file', None):
             set_monitor_status(gconf.state_file, create)
+
+        try:
+            GLogger._gsyncd_loginit(log_file=gconf.log_file, label='monitor')
+            gconf.log_exit = False
+            logging.info("Monitor Status: %s" % create)
+        except IOError:
+            if sys.exc_info()[1].errno == ENOENT:
+                # If log dir not present
+                pass
+            else:
+                raise
         return
 
     go_daemon = rconf['go_daemon']
@@ -697,8 +730,14 @@ def main_i():
 
     status_get = rconf.get('status_get')
     if status_get:
+        master_name, slave_data = get_master_and_slave_data_from_args(args)
         for brick in gconf.path:
-            brick_status = GeorepStatus(gconf.state_file, brick,
+            brick_status = GeorepStatus(gconf.state_file,
+                                        gconf.local_node,
+                                        brick,
+                                        gconf.local_node_id,
+                                        master_name,
+                                        slave_data,
                                         getattr(gconf, "pid_file", None))
             checkpoint_time = int(getattr(gconf, "checkpoint", "0"))
             brick_status.print_status(checkpoint_time=checkpoint_time)
@@ -713,7 +752,7 @@ def main_i():
     if be_monitor:
         label = 'monitor'
     elif be_agent:
-        label = 'agent'
+        label = gconf.local_path
     elif remote:
         # master
         label = gconf.local_path
@@ -730,7 +769,6 @@ def main_i():
     if be_monitor:
         return monitor(*rscs)
 
-    logging.info("syncing: %s" % " -> ".join(r.url for r in rscs))
     if remote:
         go_daemon = remote.connect_remote(go_daemon=go_daemon)
         if go_daemon:
@@ -739,6 +777,7 @@ def main_i():
             remote.connect_remote(go_daemon='done')
     local.connect()
     if ffd:
+        logging.info ("Closing feedback fd, waking up the monitor")
         os.close(ffd)
     local.service_loop(*[r for r in [remote] if r])
 
